=begin
= class NumRu::Misc::KeywordOpt

== Overview

A class to facilitate optional keyword arguments. More specifically,
it helps the use of a Hash to mimic the keyword argument system.
With this, you can set default values and description to each 
keyword argument.

== Classes defined supplementarilly

=== class NumRu::Misc::HelpMessagingException < StandardError

This is for your convenience. See the usage example below.

== Usage example

Suppose that you introduce keyword arguments "flag" and "number"
to the method "hoge" in a class/module Foo. It can be done as 
follows:

  require 'numru/misc'  # or, specifically, require 'numru/misc/keywordopt'
  include NumRu

  class Foo
    @@opt_hoge = Misc::KeywordOpt.new(
      ['flag',   false, 'whether or not ...'],
      ['number', 1,     'number of ...'],
      ['help',   false, 'show help message']
    )
    def hoge(regular_arg1, regular_arg2, options=nil)
      opt = @@opt_hoge.interpret(options)
      if opt['help']
        puts @@opt_hoge.help
        puts ' Current values='+opt.inspect
        raise Misc::HelpMessagingException, '** show help message and raise **'
      end
      # do what you want below 
      # (options are set in the Hash opt: opt['flag'] and opt['number'])
    end
  end

Here, the options are defined in the class variable @@opt_hoge
with option names, default values, and descriptions (for help 
messaging). One can use the method hoge as follows:

  foo = Foo.new
  ...
  x = ...
  y = ...
  ...
  foo.hoge( x, y, {'flag'=>true, 'number'=>10} )

Or equivalently,

  foo.hoge( x, y, 'flag'=>true, 'number'=>10 )

because '{}' can be omitted here. 

Tails of options names can be shortened as long as unambiguous:

  foo.hoge( x, y, 'fla'=>true, 'num'=>10 )


To show the help message, call

  foo.hoge( x, y, 'help'=>true )

This will cause the following help message printed with the 
exception HelpMessagingException raised.

  \** Description of options **
    option name => default value (description), ..:
  {  "flag" => false (whether or not ...),
    "number" => 1 (number of ...),
    "help" => false (show help message)}
   Current values={"help"=>true, "number"=>1, "flag"=>false}
  NumRu::Misc::HelpMessagingException: ** show help message and raise **
          from (irb):78:in "hoge"
          from (irb):83


== Class methods

---KeywordOpt.new( *args )

    Constructor.

    ARGUMENTS
    * args : arrays of two or three elements: [option name, default value,
      description ], or [option name, default value] if you do not want to
      write descriptions. Option names and descriptions must be String.

    RETURN VALUE
    * a KeywordOpt object

== Methods
---interpret(hash)

    Interprets a hash that specifies option values.

    ARGUMENTS
    * hash (Hash or nil) : a hash with string keys matching option names 
      (initializedwhen constructed). The matching is case sensitive and done
      such that the tail of a option name can be omitted as long as 
      unambiguous (for example, 'num' for 'number'). 
      If the argument is nil, the current values are returned.

    RETURN VALUE
    * a Hash containing the option values (default values overwritten
      with hash).

    POSSIBLE EXCEPTION
    * hash has a key that does not match any of the option names.
    * hash has a key that is ambiguous

---set(hash)

    Similar to ((<interpret>)) but changes internal values.

    ARGUMENTS
    * hash (Hash) : see ((<interpret>)). (Here, nil is not permitted though)

    RETURN VALUE
    * a Hash containing the values replaced (keys are the option names)

    POSSIBLE EXCEPTION
    * the argument is not a Hash
    * others are same as in ((<interpret>))

---help

    Returns a help message

    RETURN VALUE
    * a String describing the option names, default values, and descriptions

=end

module NumRu

  module Misc
    class HelpMessagingException < StandardError
    end

    class KeywordOpt
      def initialize(*args)
	# USAGE:
	#    KeywordOpt.new([key,val,description],[key,val,description],..)
	#    where key is a String, and description can be omitted.
	@val=Hash.new
	@description=Hash.new
	@keys = []
	args.each{ |x|
	  @keys.push(x[0])
	  @val[x[0]]=x[1]
	  @description[x[0]]= ( (x.length>=3) ? x[2] : '' )
	}
	@keys_sort = @keys.sort
      end

      def interpret(hash)
	return @val.dup if hash.nil?
	##
	len = @val.length
	im = 0
	out = @val.dup
	hash.keys.sort.each do |key|
	  rkey = /^#{key}/
	  loop do
	    if rkey =~ @keys_sort[im]
	      if im<len-1 && rkey=~@keys_sort[im+1]
		raise "Ambiguous key specification '#{key}'." 
	      end
	      out[@keys_sort[im]]=hash[key]
	      break
	    end
	    im += 1
	    raise "'#{key}' does not match any of the keys." if im==len
	  end
	end
	out
      end

      def set(hash)
	raise ArgumentError, "not a hash" if !hash.is_a?(Hash)
	##
	replaced = Hash.new
	len = @val.length
	im = 0
	hash.keys.sort.each do |key|
	  rkey = /^#{key}/
	  loop do
	    if rkey =~ @keys_sort[im]
	      if im<len-1 && rkey=~@keys_sort[im+1]
		raise "Ambiguous key specification '#{key}'." 
	      end
	      replaced[@keys_sort[im]] = @val[@keys_sort[im]]
	      @val[@keys_sort[im]]=hash[key]
	      break
	    end
	    im += 1
	    raise "'#{key}' does not match any of the keys." if im==len
	  end
	end
	replaced
      end
      
      def help
	"** Description of options **\n" +
	 "  option name => default value (description), ..:\n{" +
	 @keys.collect{|k| "  #{k.inspect} => #{@val[k].inspect} (#{@description[k]})"}.join(",\n") +
	 '}'
      end
    end

  end
end

if __FILE__ == $0
  include NumRu

  class Foo
    @@opt_hoge = Misc::KeywordOpt.new(
      ['flag',   false, 'whether or not ...'],
      ['number', 1,     'number of ...'],
      ['fff',    1,     'fff...'],
      ['help',   false, 'show help message']
    )
    def self.change_default(hash)
      @@opt_hoge.set(hash)
    end
    def hoge(regular_arg1, regular_arg2, options=nil)
      opt = @@opt_hoge.interpret(options)
      if opt['help']
        puts @@opt_hoge.help
        puts ' Current values='+opt.inspect
        raise Misc::HelpMessagingException, '** show help message and raise **'
      end
      # do what you want below 
      # (options are set in the Hash opt: opt['flag'] and opt['number'])
      p opt
    end
  end

  foo = Foo.new
  x = 1
  y = 1
  print "### 0 ###\n"
  foo.hoge( x, y, {'flag'=>true, 'number'=>10} )
  foo.hoge( x, y )
  print "### 1 ###\n"
  foo.hoge( x, y, 'fla'=>true, 'num'=>10 )
  print "### 2 ###\n"
  begin
    foo.hoge( x, y, 'help'=>true )
  rescue
    puts $!
  end
  print "### 3 ###\n"
  Foo.change_default( {'number'=>3} )
  begin
    foo.hoge( x, y, 'fla'=>true, 'num'=>10, 'help'=>true)
  rescue
    puts $!
  end
  print "### 4 ###\n"
  begin
    foo.hoge( x, y, 'dummy'=>nil)
  rescue
    puts $!
  end
  print "### 5 ###\n"
  begin
    foo.hoge( x, y, 'f'=>nil)
  rescue
    puts $!
  end
end
